\name{DandD}
\alias{DandD}
\title{Import DandD Instance into \R}
\description{
   Function to import a DandD instance into \R as a \code{dad} object
}
\usage{
   DandD(file="", prefix="http://www.stat.math.keio.ac.jp/DandDIII/Examples/",
			 radix=FALSE, language="English")
}
\arguments{
  \item{file}{character. the name of a DandD instance. Possible examples of DandD instances 
are shown by default.}
  \item{prefix}{character.  URL for the file.}
  \item{radix}{logical,  indicatin if any radix system should
  be kept as is.  
  If ``TRUE'',  any radix system is not resolved into a vector  }
  \item{language}{character. Choice of language (currently ignored).}
}
\value{
 \code{dad} object.
}
\details{
\code{DandD} loads a DandD instance and returns a \code{dad} object.
The \code{dad} object is a list of every components of <Data>  in the DandD instance.
At loading,  any data vector in the DandD instance is transformed into
a function call,  so that application of the function \code{eval} is necessary 
to get the actual values.

The \code{dad} object returned by this function can be transformed into several dataframes
by applying the function \code{data.frame} to each element of the \code{dad} object 
as shown in the following example, or attached to the R search path
by using \code{attachDandD} as a promise object, that is, a delayed evaluation object.
The latter mechanism is automatically invoked by the print method 
\code{print.dad} for \code{dad} object.

\code{dad} object has its own \code{\link{plot}} method which
produces a Textile Plot of relational data in the object.
see also \code{plot.dad}.
}
\seealso{
 \code{\link{openConn}}
 \code{\link{closeConn}}
 \code{\link{radix}}
}

\examples{
acid=DandD("AcidRain.dad")   # import a DandD instance and assign it to acid.
acid                         # print the dad object automatically attached.
acid$AcidRan$SlftCcn         # call object
eval(acid$AcidRan$SlftCcn)   # get actual values.
SlftCcn                      # access the promise object attached to the R search path.
acid.d=data.frame(acid[[1]]) # coerce to data frame.
acid.d
plot(acid)                   # produce plots
}
\keyword{dad}
\keyword{DandD}
\keyword{open}
