#include <stdio.h>
#include "blaswrap.h"
#include "f2c.h"
#include "clapack.h"
#include "transform.h"

#define TINY 1.0e-10





void printVector(integer *m, integer ln, char* cas){
	integer i;
	for(i=0; i<ln; i++){
		printf("%ld", m[i]);
		printf(" ");
	}
	printf("\n");
}

void printVector2(double* m, integer ln, char* cas){
	integer i;
	for(i=0; i<ln; i++){
		printf("%f", m[i]);
		printf(" ");
	}
	printf("\n");
}



void printMatrix(double* m, integer n, integer p, integer ln){
	integer i, j;
	for(i=0; i<n; i++){
		for(j=0; j<p; j++){
			if(n==p){
				if(j>=i){
					printf("%.5f ", m[j*ln+i]);
				}else{
					printf("*\t");
				}
			}else{
				printf("%.5f ", m[j*ln+i]);
			}
		}
		printf("\n");
	}
	printf("\n");
}


/* sample main end */


void subMatrix(integer p, double* a, double* b, integer* q){//a is a p x p matrix.
	integer i, j, k, l, r, s;
	l=0;
	r=0;
	
	for(i=0; i<p; i++){
		if(q[l] == i){
			l++;
		}else{
			k=0;
			s=0;
			for(j=0; j<=i; j++){
				if(q[k] == j){
					k++;
				}else{
					b[r*p+s] = a[i*p+j];
					s++;
				}
			}
			r++;
		}
	}
}

void superVector(integer Q, double* a, double* b, integer* q){
	integer k, l, i;
	l=0;
	k=0;
	for(i=0; i<Q; i++){
		if(q[l] == i){
			b[i] = 0.0;
			l++;
		}else{
			b[i] = a[k];
			k++;
		}
	}
}



double sum(double* a, integer m){
	double res=0;
	integer i;
	for(i=0; i<m; i++){
		res += a[i];
	}
	return(res);
}


double ip(double* a, double* b, integer m){
	double res=0;
	integer i;
	for(i=0; i<m; i++){
		res += a[i]*b[i];
	}
	return(res);
}
double wip(double* a, double* b, double* c, integer m){
	double res=0;
	integer i;
	for(i=0; i<m; i++){
		if(c[i] > 0.0){
			res += a[i]*b[i]/c[i];
		}
	}
	return(res);
}

void ginv(double* ainv, integer n){
	integer i, j, k;
	char jobz='V';
	char uplo='U';
	integer nn = n;
	integer lda = (integer)n;
	double w[n+1];
	double work[3*n];
	double a[n*n+1];
	integer lwork;
	integer info;
	
	//lda=n;
	//lwork=3*nn;
	(lwork) = 3*nn;
	for(i=0; i<n*n; i++){
		a[i] = ainv[i];
		ainv[i] = 0.0;
	}
	//printf("hoge=%ld %ld\n",lda, nn);
	//printMatrix(a, n ,n ,n);
	dsyev_(&jobz, &uplo, (integer *) &nn, a, (integer *) &nn, w, work, &lwork, (integer *) &info);
	//printMatrix(a, n, 5, n);
	//printf("info=%ld\n", info);
	for(i=0; i<n; i++){
		if(w[i]>TINY){
			for(j=0; j<n; j++){
				for(k=0; k<=j; k++){
					ainv[j*n+k] += a[i*n+j]*a[i*n+k]/w[i];
				}
			}
		}
	}
	//printMatrix(ainv, n ,n ,n);
}

void ca(double* a11, double* a12, double* a22, integer p, integer Q){
	integer i, j, k;
	double temp[p*Q+1];
	double temp2;
	
	for(i=0; i<Q; i++){
		for(j=0; j<p; j++){
			temp[i*p+j] = 0.0;
			for(k=0; k<p; k++){
				if(j>k){
					temp[i*p+j] += a11[j*p+k]*a12[i*p+k];
				}else{
					temp[i*p+j] += a11[k*p+j]*a12[i*p+k];
				}
			}
		}
	}
	//printMatrix(temp, p, Q, p);
	for(i=0; i<Q; i++){
		for(j=0; j<=i; j++){
			temp2 = 0.0;
			for(k=0; k<p; k++){
				temp2 += a12[j*p+k]*temp[i*p+k];
			}
			a22[i*Q+j] = temp2 - a22[i*Q+j];
		}
	}
	for(i=0; i<p*Q; i++){
		a12[i] = temp[i];
	}
}

int Dsygv1(double* a, double* b, integer Q, integer R, double* beta, integer dim, double* mev){
	integer i;
	integer itype = 1;
	char jobz = 'V';
	char uplo = 'U';
	double ev[Q];
	
	integer lda = Q;
	integer ldb = Q;
	integer pp = R;
	double work[3*Q];
	integer lwork = 3*Q;
	integer info=0;
	
	double suba[Q*Q+1];
	double subb[Q*Q+1];
	
	for(i=0; i<Q*R; i++){
		suba[i] = a[i];
		subb[i] = b[i];
	}
	//printMatrix(subb, 5,5,Q);
	
	//printf("dsyev1\n");
	
	info = dsygv_((integer *) &itype, &jobz, &uplo, (integer *) &pp, suba, (integer *) &lda, subb, (integer *) &ldb, ev, work, (integer *) &lwork, (integer *) &info);
	
	
	//printVector2(ev, R, "%f");
	//printf("\n\n");
	//printf("maxev=%f\n", ev[Q-dim]);
	(*mev) = ev[R-dim];
	dim = Q*(R-dim);
	for(i=0; i<R; i++){
		beta[i] = suba[dim + i];
	}
	
	//printf("ev=");
	//printVector2(beta, Q, "%f");
	//printf("info=%ld\n", info);
	return info;
}



int satisfy(double* ev, integer* q, integer lo, integer* ordered){
	integer i, j;
	integer resultN;
	integer resultP;
	
	resultN = 1;
	resultP = 1;
	i=0;
	while((resultP+resultN) > 0 && i < lo){
		j = q[ordered[i]-1];
		resultN = 1;
		resultP = 1;
		while( (resultN+resultP) > 0 && j < q[ordered[i]] ){
			if(ev[j] > 0){
				resultN*=0;
				resultP*=1;
			}else if(ev[j] < 0){
				resultN*=1;
				resultP*=0;
			}else{
				resultN*=1;
				resultP*=1;
			}
			j++;
		}
		i++;
	}
	return (resultN+resultP);
}



int Dsygv2(double* a, double* b, integer* k, integer* q, integer Q, integer R, double* maxev, double* beta, integer lo, integer* ordered, integer* orderedOrig, integer* orderedFlag, integer lc, integer dim, integer* eflag,
integer* itype, char* jobz, char* uplo, integer* lda, integer* ldb, integer* lwork, integer* info,
double* work, double* ev, double* suba, double* subb, integer* orderedCol, double* betaTemp, integer* depth){
	
	integer pp = (R-(*k));
	integer i, j;
	//printf("%ld ", *k);
	j=0;
	for(i=0; i<lc; i++){
		if(orderedFlag[i] > 0){
			orderedCol[j] = orderedOrig[i];
			j++;
		}
	}
	
	for(i=0; i<Q; i++){
		ev[i] = 0.0;
	}
	
	
	orderedCol[*k] = -1;
	subMatrix(Q, a, suba, orderedCol);
	subMatrix(Q, b, subb, orderedCol);
	if(pp>0){
		(*info) = dsygv_(itype, jobz, uplo, &pp, suba, lda, subb, ldb, ev, work, lwork, info);
		superVector(R, &(suba[Q*(pp-dim)]), betaTemp, orderedCol);
	}
	
	//printVector(orderedFlag, lc, "%d");
	//printf("%f", ev[pp-dim]);
	if(pp > 0){
		if(satisfy(betaTemp, q, lo, ordered) > 0 && (*maxev) < ev[pp-dim]){
			for(i=0; i<R; i++){
				beta[i] = betaTemp[i];
			}
			(*maxev) = ev[pp-dim];
			//printf("matrix=%ld\n",pp);
			//printMatrix(subb, pp, pp, pp);
			//printMatrix(suba, pp, pp, pp);
			
			//printf("info=%ld\n",*info);
			//printVector(orderedFlag, R, "%d");
			//printf("maxev = %f\n", *maxev);
			
			(*depth) = R;//(*k)+2;
		}
	}
	if((*k)==0){
		//printf("eflag=%ld\n", *eflag);
		orderedFlag[lc-1]=1;
		if((*eflag)==1){
			return (*info);
		}else{
			(*eflag)=1;
			(*k)=1;
			Dsygv2(a, b, k, q, Q, R, maxev, beta, lo, ordered, orderedOrig, orderedFlag, lc, dim, eflag,
			itype, jobz, uplo, lda, ldb, lwork, info, 
			work, ev, suba, subb, orderedCol, betaTemp, depth);
		}
	}else if(orderedFlag[0] == 1){
		orderedFlag[0] = 0;
		i=0;
		while(orderedFlag[i] < 1){
			i++;
		}
		orderedFlag[i-1]=1;
		orderedFlag[i] = 0;
		(*k)--;;
		return (*info);
	}else{
		if((*maxev) > ev[pp-dim] && *depth < (*k)){
			i=0;
			while(orderedFlag[i] < 1){
				i++;
			}
			orderedFlag[i]=0;
			orderedFlag[i-1]=1;
			return (*info);
			//Dsygv2(a, b, k, q, Q, R, maxev, beta, lo, ordered, orderedOrig, orderedFlag, lc, dim, eflag,
			//itype, jobz, uplo, lda, ldb, lwork, info,
			//work, ev, suba, subb, orderedCol, betaTemp, depth);
		}else{
			i=0;
			while(orderedFlag[i] < 1){
				i++;
			}
			orderedFlag[i-1]=1;
			(*k)++;
			Dsygv2(a, b, k, q, Q, R, maxev, beta, lo, ordered, orderedOrig, orderedFlag, lc, dim, eflag,
			itype, jobz, uplo, lda, ldb, lwork, info,
			work, ev, suba, subb, orderedCol, betaTemp, depth);
		}
	}
	Dsygv2(a, b, k, q, Q, R, maxev, beta, lo, ordered, orderedOrig, orderedFlag, lc, dim, eflag,
		itype, jobz, uplo, lda, ldb, lwork, info,
		work, ev, suba, subb, orderedCol, betaTemp, depth);
	return (*info);
}

double Abs(double value){
	if(value<0){
		value *= -1;
	}
	return value;
}

int gradient(double* a, double* b, integer Q, integer R, double* maxev, double* beta, double* grad){
	integer i, j;
	for(i=0; i<R; i++){
		grad[i] = 0.0;
		if(beta[i] == 0.0){
			for(j=0; j<R; j++){
				if(i > j){
					grad[i] = grad[i] - a[Q*i+j]*beta[j] + (*maxev)*b[Q*i+j]*beta[j];
				}else{
					grad[i] = grad[i] - a[Q*j+i]*beta[j] + (*maxev)*b[Q*j+i]*beta[j];
				}
			}
		}
	}
	return 1;
}
int Dsygv3(double* a, double* b, integer* q, integer Q, integer R, double* maxev, double* beta, integer lo, integer* ordered, 
integer* orderedOrig, integer* orderedFlag, integer lc, integer dim){
	
	integer itype = 1;
	char jobz = 'V';
	char uplo = 'U';
	double ev[Q];
	
	integer lda = Q;
	integer ldb = Q;
	
	double work[3*Q];
	integer lwork = 3*Q;
	integer info=0;
	
	integer rec=0;
	integer orderedCol[Q];
	integer pp = R;
	integer i, j, k, l;
	double grad[Q+1];
	double sign[Q+1];
	integer gflag;
	
	double suba[Q*Q+1];
	double subb[Q*Q+1];
	
	integer orderedFlagOld[Q+1];
	integer orderedFlagOld2[Q+1];
	for(i=0; i<Q; i++){
		orderedFlagOld[i] = 1;
		orderedFlagOld2[i] = 1;
	}
	integer oflag;
	integer oflag2;
	
	printVector(orderedFlag, lc, "%d");
	printVector2(beta, R, "%f");
	printf("maxev=%f\n", (*maxev));
	while(1){
		rec++;
		while(satisfy(beta, q, lo, ordered) == 0){ // create a beta satisfing the second constraint.
			//printf("%d", rec);
			for(i=0; i<Q; i++){
				ev[i] = 0.0;
			}
			k=0;
			l=0;
			for(i=0; i<lo; i++){
				sign[i] = 0.0;
				for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
					if(orderedFlag[l] == 0){
						sign[i] += beta[j]*Abs(beta[j]);
					}
					//printf("%f ", beta[j]);
					l++;
				}
				//printf("\nsign=%f\n", sign[i]);
				for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
					if(sign[i] > 0.0 && beta[j] < 0.0){
						orderedFlag[k] = 1;
					}else if(sign[i] < 0.0 && beta[j] > 0.0){
						orderedFlag[k] = 1;
					}else{
						//orderedFlag[k] = 0;
					}
					k++;
				}
			}
			//printVector(orderedFlag, lc, "%d");
			j=0;
			for(i=0; i<lc; i++){
				if(orderedFlag[i] > 0){
					orderedCol[j] = orderedOrig[i];
					j++;
				}
			}
			//printf("j=%d\n",j);
			orderedCol[j] = -1;
			pp = R-j;
			//printVector(orderedCol, j+1, "%d");
			
			subMatrix(Q, a, suba, orderedCol);
			subMatrix(Q, b, subb, orderedCol);
			
			//printMatrix(b, 5, 5, Q);
			//printMatrix(subb, 5, 5, Q);
			
			if(pp>0){
				//printf("stapp=%d\n", pp);
				//printf("lda=%d", lda);
				//printf("R=%d", R);
				dsygv_(&itype, &jobz, &uplo, &pp, suba, &lda, subb, &ldb, ev, work, &lwork, &info);
				if(info > 0){
					//printf("info=%d\n", info);
					getchar();
				}
				superVector(R, &(suba[Q*(pp-dim)]), beta, orderedCol);
				(*maxev) = ev[pp-dim];
			}
			//printf("ev=");
			//printVector2(ev, pp, "%f");
			//getchar();
		}
		//printf("maxev=%f\n", (*maxev));
		
		oflag=1;
		oflag2=1;
		for(i=0; i<lc; i++){
			if(orderedFlag[i] == orderedFlagOld[i]){
				oflag *= 1;
			}else{
				oflag *= 0;
			}
			if(orderedFlag[i] == orderedFlagOld2[i]){
				oflag2 *= 1;
			}else{
				oflag2 *= 0;
			}
			orderedFlagOld[i] = orderedFlag[i];
			orderedFlagOld2[i] = orderedFlagOld[i];
		}
		if(oflag2 > 1){
			oflag = 1;
		}
		if(oflag>0){
			//printf("oflag=%d\n", oflag);
			//getchar();
		}
		//####################### GRADIENT ###############################
		gradient(a, b, Q, R, maxev, beta, grad);
		k=0;
		l=0;
		gflag=0;
		//printVector(orderedFlag, lc, "%d");
		//printVector2(beta, R, "%f");
		//printVector2(grad, R, "%f");
		for(i=0; i<lo; i++){
			for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
				if(orderedFlag[l] == 0){
					sign[i] += beta[j];
				}
				l++;
			}
			//printf("\nsign=%f\n", sign[i]);
			for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
				//printf("%f ",beta[j]);
				//printf("%f ",grad[j]);
				if(orderedFlag[k] > 0){
					if(sign[i] > 0.0 && grad[j] < 0.0){
						orderedFlag[k] = 0;
						gflag++;
					}else if(sign[i] < 0.0 && grad[j] > 0.0){
						orderedFlag[k] = 0;
						gflag++;
					}
				}
				k++;
			}
		}
		//printVector(orderedFlag, lc, "%ld");
		//printf("gflag=%ld", gflag);
		//printf("maxev=%f\n", (*maxev));
		//getchar();
		if(gflag == 0){
			return 1;
		}else if(rec > 1000){
			printf("Not converge!\n");
			break;
		}else{
			j=0;
			if(oflag==0){
				for(i=0; i<lc; i++){
					if(orderedFlag[i] > 0){
						orderedCol[j] = orderedOrig[i];
						j++;
					}
				}
			}else{
				for(i=0; i<lc; i++){
					if(orderedFlag[i] > 0){
						orderedFlag[i] = 0;
					}
				}
			}
			orderedCol[j] = -1;
			
			pp = R-j;
			//printf("endpp=%ld\n",pp);
			subMatrix(Q, a, suba, orderedCol);
			subMatrix(Q, b, subb, orderedCol);
			if(pp>0){
				dsygv_(&itype, &jobz, &uplo, &pp, suba, &lda, subb, &ldb, ev, work, &lwork, &info);
				superVector(R, &(suba[Q*(pp-dim-oflag)]), beta, orderedCol);
				(*maxev) = ev[pp-dim-oflag];
				if(info > 0){
					//printf("info=%ld\n", info);
				}
			}
		}
	}
	return 0;
}

























int Dsygv4(double* a, double* b, integer* q, integer Q, integer R, double* maxev, double* beta, integer lo, integer* ordered, 
integer* orderedOrig, integer* orderedFlag, integer lc, integer dim){
	
	integer itype = 1;
	char jobz = 'V';
	char uplo = 'U';
	double ev[Q];
	
	integer lda = Q;
	integer ldb = Q;
	
	double work[3*Q];
	integer lwork = 3*Q;
	integer info=0;
	
	integer rec=0;
	integer orderedCol[Q];
	integer pp = R;
	integer i, j, k, l;
	double grad[Q+1];
	double sign[Q+1];
	integer gflag;
	
	double suba[Q*Q+1];
	double subb[Q*Q+1];
	
	integer orderedFlagOld[Q+1];
	integer orderedFlagOld2[Q+1];
	for(i=0; i<Q; i++){
		orderedFlagOld[i] = 1;
		orderedFlagOld2[i] = 1;
	}
	integer oflag;
	integer oflag2;
	
	//printVector(orderedFlag, lc, "%ld");
	//printVector2(beta, R, "%f");
	//printf("maxev=%f\n", (*maxev));
	
	while(1){
		rec++;
		while(satisfy(beta, q, lo, ordered) == 0){ // create a beta satisfing the second constraint.
			//printf("%ld", rec);
			for(i=0; i<Q; i++){
				ev[i] = 0.0;
			}
			k=0;
			l=0;
			for(i=0; i<lo; i++){
				sign[i] = 0.0;
				for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
					if(orderedFlag[l] == 0){
						sign[i] += beta[j]*Abs(beta[j]);
					}
					//printf("%f ", beta[j]);
					l++;
				}
				//printf("\nsign=%f\n", sign[i]);
				for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
					if(sign[i] > 0.0 && beta[j] < 0.0){
						orderedFlag[k] = 1;
					}else if(sign[i] < 0.0 && beta[j] > 0.0){
						orderedFlag[k] = 1;
					}else{
						//orderedFlag[k] = 0;
					}
					k++;
				}
			}
			//printVector(orderedFlag, lc, "%ld");
			j=0;
			for(i=0; i<lc; i++){
				if(orderedFlag[i] > 0){
					orderedCol[j] = orderedOrig[i];
					j++;
				}
			}
			//printf("j=%ld\n",j);
			orderedCol[j] = -1;
			pp = R-j;
			//printVector(orderedCol, j+1, "%ld");
			
			subMatrix(Q, a, suba, orderedCol);
			subMatrix(Q, b, subb, orderedCol);
			
			//printMatrix(b, 5, 5, Q);
			//printMatrix(subb, 5, 5, Q);
			
			if(pp>0){
				//printf("stapp=%ld\n", pp);
				//printf("lda=%ld", lda);
				//printf("R=%ld", R);
				dsygv_(&itype, &jobz, &uplo, &pp, suba, &lda, subb, &ldb, ev, work, &lwork, &info);
				if(info > 0){
					//printf("info=%ld\n", info);
					getchar();
				}
				superVector(R, &(suba[Q*(pp-dim)]), beta, orderedCol);
				(*maxev) = ev[pp-dim];
			}
			//printf("ev=");
			//printVector2(ev, pp, "%f");
			//getchar();
		}
		//printf("maxev=%f\n", (*maxev));
		
		oflag=1;
		oflag2=1;
		for(i=0; i<lc; i++){
			if(orderedFlag[i] == orderedFlagOld[i]){
				oflag *= 1;
			}else{
				oflag *= 0;
			}
			if(orderedFlag[i] == orderedFlagOld2[i]){
				oflag2 *= 1;
			}else{
				oflag2 *= 0;
			}
			orderedFlagOld[i] = orderedFlag[i];
			orderedFlagOld2[i] = orderedFlagOld[i];
		}
		if(oflag2 > 1){
			oflag = 1;
		}
		oflag=0;
		if(oflag>0){
			//printf("oflag=%ld\n", oflag);
			//getchar();
		}
		//####################### GRADIENT ###############################
		gradient(a, b, Q, R, maxev, beta, grad);
		k=0;
		l=0;
		gflag=0;
		//printVector(orderedFlag, lc, "%ld");
		//printVector2(beta, R, "%f");
		//printVector2(grad, R, "%f");
		for(i=0; i<lo; i++){
			for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
				if(orderedFlag[l] == 0){
					sign[i] += beta[j];
				}
				l++;
			}
			//printf("\nsign=%f\n", sign[i]);
			for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
				//printf("%f ",beta[j]);
				//printf("%f ",grad[j]);
				if(orderedFlag[k] > 0){
					if(sign[i] > 0.0 && grad[j] < 0.0){
						orderedFlag[k] = 0;
						gflag++;
					}else if(sign[i] < 0.0 && grad[j] > 0.0){
						orderedFlag[k] = 0;
						gflag++;
					}
				}
				k++;
			}
		}
		//printVector(orderedFlag, lc, "%ld");
		printf("gflag=%ld", gflag);
		//printf("maxev=%f\n", (*maxev));
		//getchar();
		if(gflag == 0){
			//printf("res=%ld\n", rec);
			return 1;
		}else if(rec > 1000){
			printf("Not converge!\n");
			break;
		}else{
			j=0;
			if(oflag==0){
				for(i=0; i<lc; i++){
					if(orderedFlag[i] > 0){
						orderedCol[j] = orderedOrig[i];
						j++;
					}
				}
			}else{
				for(i=0; i<lc; i++){
					if(orderedFlag[i] > 0){
						orderedFlag[i] = 0;
					}
				}
			}
			orderedCol[j] = -1;
			
			pp = R-j;
			//printf("endpp=%ld\n",pp);
			subMatrix(Q, a, suba, orderedCol);
			subMatrix(Q, b, subb, orderedCol);
			if(pp>0){
				dsygv_(&itype, &jobz, &uplo, &pp, suba, &lda, subb, &ldb, ev, work, &lwork, &info);
				superVector(R, &(suba[Q*(pp-dim)]), beta, orderedCol);
				(*maxev) = ev[pp-dim];
				if(info > 0){
					//printf("info=%ld\n", info);
				}
			}
		}
	}
	return 0;
}














int Dsygv5(double* a, double* b, integer* q, integer Q, integer R, double* maxev, double* beta, integer lo, integer* ordered, 
integer* orderedOrig, integer* orderedFlag, integer lc, integer dim){
	
	integer itype = 1;
	char jobz = 'V';
	char uplo = 'U';
	double ev[Q];
	
	integer lda = Q;
	integer ldb = Q;
	
	double work[3*Q];
	integer lwork = 3*Q;
	integer info=0;
	
	integer rec;
	integer rec2;
	integer orderedCol[Q];
	integer pp = R;
	integer i, j, k, l, f, kk;
	double grad[Q+1];
	double sign[Q+1];
	double sign2[Q+1];
	integer gflag;
	integer bflag;
	
	double suba[Q*Q+1];
	double subb[Q*Q+1];
	
	integer orderedFlagOld[Q+1];
	integer orderedFlagOld2[Q+1];
	for(i=0; i<Q; i++){
		orderedFlagOld[i] = 1;
		orderedFlagOld2[i] = 1;
	}
	integer oflag;
	integer oflag2;
	
	//printVector(orderedFlag, lc, "%ld");
	//printVector2(beta, R, "%f");
	//printf("maxev=%f\n", (*maxev));
	rec=0;
	rec2=0;
	do{ // create a beta satisfing the second constraint.
		rec++;
		gflag=0;
		bflag=0;
		for(i=0; i<Q; i++){
			ev[i] = 0.0;
		}
		j=0;
		for(i=0; i<lc; i++){
			if(orderedFlag[i] > 0){
				orderedCol[j] = orderedOrig[i];
				j++;
			}
		}
		orderedCol[j] = -1;
		pp = R-j;
		
		subMatrix(Q, a, suba, orderedCol);
		subMatrix(Q, b, subb, orderedCol);
		dsygv_(&itype, &jobz, &uplo, &pp, suba, &lda, subb, &ldb, ev, work, &lwork, &info);
		if(info > 0){
			getchar();
		}
		superVector(R, &(suba[Q*(pp-dim)]), beta, orderedCol);
		(*maxev) = ev[pp-dim];
		printf("maxev=%f\n", (*maxev));
		gradient(a, b, Q, R, maxev, beta, grad);
		
		k=0;
		l=0;
		f=0;
		for(i=0; i<lo; i++){
			sign[i] = 0.0;
			sign2[i]= 0.0;
			for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
				for(kk=q[ordered[i]-1]; kk<q[ordered[i]]; kk++){
					if(beta[j] > 0.0 && beta[kk] > 0.0){ // beta+
						if(j > kk){
							sign[i] += beta[j]*b[kk+j*Q]*beta[kk];
						}else{
							sign[i] += beta[j]*b[j+kk*Q]*beta[kk];
						}
					}else if(beta[j] < 0.0 && beta[kk] < 0.0){ // beta-
						if(j > kk){
							sign[i] -= beta[j]*b[kk+j*Q]*beta[kk];
						}else{
							sign[i] -= beta[j]*b[j+kk*Q]*beta[kk];
						}
					}
					//sign[i] += beta[j]*Abs(beta[j]);
					l++;
				}
				sign2[i] += beta[j]*Abs(beta[j]);
			}
			if(sign[i]==0.0){
					printf("zero!");
					printf("%d\n",i);
					sign[i] = sign2[i];
			}
			for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
				if(sign[i] > 0.0 && beta[j] < 0.0){
					orderedFlag[k] = 1;
					bflag++;
				}else if(sign[i] < 0.0 && beta[j] > 0.0){
					orderedFlag[k] = 1;
					bflag++;
				}
				k++;
			}
			for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
				//if(orderedFlag[f] > 0){
					if(sign[i] > 0.0 && grad[j] < 0.0){
						orderedFlag[f] = 0;
						gflag++;
					}else if(sign[i] < 0.0 && grad[j] > 0.0){
						orderedFlag[f] = 0;
						gflag++;
					}
				//}
				f++;
			}
		}
		if(rec > 100){
			//printf("%ld Not converge!\n", gflag);
			if(bflag == 0 || rec2 > 300){
				printf("Sorry, not converged!\n", gflag);
				break;
			}else{
				
				rec2 += rec;
				rec=0;
				k=0;
				l=0;
				f=0;
				for(i=0; i<lo; i++){
					for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
						if(sign[i] > 0.0 && beta[j] < 0.0){
							orderedFlag[k] = 0;
						}else if(sign[i] < 0.0 && beta[j] > 0.0){
							orderedFlag[k] = 0;
						}
						k++;
					}
					for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
						if(sign[i] > 0.0 && beta[j] > 0.0){
							//orderedFlag[f] = 1;
						}else if(sign[i] < 0.0 && beta[j] < 0.0){
							//orderedFlag[f] = 1;
						}
						f++;
					}
					
				}
				
				
				
				
				
				
				
				
				
				
			}
		}
	//}while(satisfy(beta, q, lo, ordered) == 0 || gflag > 0);
	}while(bflag > 0 || gflag > 0);
	return 0;
}




int Dsygv5old(double* a, double* b, integer* q, integer Q, integer R, double* maxev, double* beta, integer lo, integer* ordered, 
integer* orderedOrig, integer* orderedFlag, integer lc, integer dim){
	
	integer itype = 1;
	char jobz = 'V';
	char uplo = 'U';
	double ev[Q];
	
	integer lda = Q;
	integer ldb = Q;
	
	double work[3*Q];
	integer lwork = 3*Q;
	integer info=0;
	
	integer rec;
	integer rec2;
	integer orderedCol[Q];
	integer pp = R;
	integer i, j, k, l, f;
	double grad[Q+1];
	double sign[Q+1];
	integer gflag;
	integer bflag;
	
	double suba[Q*Q+1];
	double subb[Q*Q+1];
	
	integer orderedFlagOld[Q+1];
	integer orderedFlagOld2[Q+1];
	for(i=0; i<Q; i++){
		orderedFlagOld[i] = 1;
		orderedFlagOld2[i] = 1;
	}
	integer oflag;
	integer oflag2;
	
	//printVector(orderedFlag, lc, "%ld");
	//printVector2(beta, R, "%f");
	//printf("maxev=%f\n", (*maxev));
	rec=0;
	rec2=0;
	do{ // create a beta satisfing the second constraint.
		rec++;
		gflag=0;
		bflag=0;
		for(i=0; i<Q; i++){
			ev[i] = 0.0;
		}
		j=0;
		for(i=0; i<lc; i++){
			if(orderedFlag[i] > 0){
				orderedCol[j] = orderedOrig[i];
				j++;
			}
		}
		orderedCol[j] = -1;
		pp = R-j;
		
		subMatrix(Q, a, suba, orderedCol);
		subMatrix(Q, b, subb, orderedCol);
		dsygv_(&itype, &jobz, &uplo, &pp, suba, &lda, subb, &ldb, ev, work, &lwork, &info);
		if(info > 0){
			getchar();
		}
		superVector(R, &(suba[Q*(pp-dim)]), beta, orderedCol);
		(*maxev) = ev[pp-dim];
		gradient(a, b, Q, R, maxev, beta, grad);
		
		k=0;
		l=0;
		f=0;
		for(i=0; i<lo; i++){
			sign[i] = 0.0;
			for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
				sign[i] += beta[j]*Abs(beta[j]);
				l++;
			}
			if(sign[i]==0.0){
					//printf("zero!");
			}
			for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
				if(sign[i] > 0.0 && beta[j] < 0.0){
					orderedFlag[k] = 1;
					bflag++;
				}else if(sign[i] < 0.0 && beta[j] > 0.0){
					orderedFlag[k] = 1;
					bflag++;
				}
				k++;
			}
			for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
				//if(orderedFlag[f] > 0){
					if(sign[i] > 0.0 && grad[j] < 0.0){
						orderedFlag[f] = 0;
						gflag++;
					}else if(sign[i] < 0.0 && grad[j] > 0.0){
						orderedFlag[f] = 0;
						gflag++;
					}
				//}
				f++;
			}
		}
		if(rec > 1000){
			//printf("%ld Not converge!\n", gflag);
			if(bflag == 0 || rec2 > 3000){
				printf("Sorry, not converged!\n", gflag);
				break;
			}else{
				
				rec2 += rec;
				rec=0;
				k=0;
				l=0;
				f=0;
				for(i=0; i<lo; i++){
					for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
						if(sign[i] > 0.0 && beta[j] < 0.0){
							orderedFlag[k] = 0;
						}else if(sign[i] < 0.0 && beta[j] > 0.0){
							orderedFlag[k] = 0;
						}
						k++;
					}
					for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
						if(sign[i] > 0.0 && beta[j] > 0.0){
							//orderedFlag[f] = 1;
						}else if(sign[i] < 0.0 && beta[j] < 0.0){
							//orderedFlag[f] = 1;
						}
						f++;
					}
					
				}
				
				
				
				
				
				
				
				
				
				
			}
		}
	//}while(satisfy(beta, q, lo, ordered) == 0 || gflag > 0);
	}while(bflag > 0 || gflag > 0);
	return 0;
}






















//void transform(double* xx, double* ww, double* y, integer n, integer p, integer* q, integer Q, integer dim, integer* ordered, integer lo){
void transform(double* xx, double* ww, double* y, integer *nn, integer* pp, integer* q, integer* QQ, integer* ddim, integer* ordered, integer* llo, double* mev){
	
	integer n=(*nn);
	integer p=(*pp);
	integer Q=(*QQ);
	integer R=0;
	integer lo=(*llo);
	integer dim=(*ddim);
	integer eflag = 0;
	
	integer itype = 1;
	char jobz = 'V';
	char uplo = 'U';
	integer lda = Q;
	integer ldb = Q;
	integer lwork = 3*Q;
	integer info=0;
	
	double work[3*Q];
	double ev[Q+1];
	
	integer orderedCol[Q+1];
	double betaTemp[Q+1];
	
	/*
	n       (INPUT)   INTEGER
	        The number of recoreds.
	        
	p       (INPUT)   INTEGER
	        The number of variables.
	        
	q       (INPUT)   INTEGER * (p + 1)
	        (p+1)-dimensional array. Xɂ0n܂eϗʂ̐擪̗ԍ. 
	        
	Q       (INPUT)   INTEGER
	        The number of colmuns of encoded data matrix X.
	        
	xx      (INPUT)   DOUBLE * (n * Q)
	        An n x Q encoded data matrix X.
	        R[fBOꂽf[^s
	        
	ww      (INPUT)   DOUBLE * (n * p)
	        An n x p weight matrix W, reflecting missing informations.
	        0CȊO1̏dݍs
	        
	y       (OUTPUT)  DOUBLE * (n * p)
	        An n x p transformed data matrix Y.
	        
	dim     (INPUT)   INTEGER
	        The dimension you want to choose, ͂Ƃ肠1.
	        
	ordered (INPUT)   INTEGER * r
	        r{̏q܂܂ĂꍇC̕ϗʔԍzŗ^.
	        2l̏q͊܂܂ȂD
	        q܂܂ȂNULLZbgD
	        
	lo      (INPUT)   INTEGER
	        The number of ordered categorical data vectors.
	*/
	
	
	double* X[Q+1]; //X   n x Q
	double* W[p+1]; //W   n x p
	double dd[n+1]; //w = Z*t(Z)*W*1
	double d[n+1];  //w = Z*t(Z)*W*1
	
	double *x;
	x = (double *)R_alloc(n*Q+1, sizeof(double));
	double *w;
	w = (double *)R_alloc(n*p+1, sizeof(double));
	/*
	double x[n*Q+1];
	double w[n*p+1];
	*/
	
	
	
	
	double alpha;
	double diagb[Q+1];
	integer qordered[p+1];
	
	double suba[Q*Q+1];
	double subb[Q*Q+1];
	
	
	//Z   Indicator Matrix of Target Object
	double a11[p*p+1]; //A11 p x p
	double a12[p*Q+1]; //A12 p x Q
	double a22[Q*Q+1]; //A22 Q x Q
	double b[Q*Q+1]; //B
	
	integer depth=0;
	integer tolerateDepth=Q;
	double maxev=0.0;
	double beta[Q+1];
	
	
	integer i, j, k, l, m, r, s, rtemp, stemp;
	integer lc;
	double temp2, temp;
	for(i=0; i<n; i++){// making d
		dd[i] = 0;
		for(j=0; j<p; j++){
			dd[i] += ww[n*j+i];
		}
	}
	
	//printf("hoge\n");
	
	m=0;
	for(i=0; i<n; i++){
		if(dd[i]>0.0){
			for(j=0; j<p; j++){
				w[j*n+m] = ww[j*n+i];
				for(k=q[j]; k<q[j+1]; k++){
					x[k*n+m] = xx[k*n+i]*ww[j*n+i];
				}
			}
			d[m] = dd[i];
			m++;
		}
	}
	
	for(j=0; j<Q; j++){
		X[j] = &x[j*n];
	}
	for(j=0; j<p; j++){
		W[j] = &w[j*n];
	}
	
	for(j=0; j<p; j++){
		for(k=q[j]; k<q[j+1]; k++){
			diagb[k] = ip(X[k], X[k], m) - sum(X[k], m)*sum(X[k], m)/sum(W[j], m);
		}
	}
	
	R=0;
	for(j=0; j<Q; j++){
		//printf("%f ", diagb[j]);
		if(diagb[j]>0.0){
			R++;
		}
	}
	//printVector2(diagb, Q, "%f");
	//printf("R=%ld\n", R);
	temp=0.0;
	r=0;
	s=0;
	rtemp=0;
	stemp=0;
	
	
	
	
	
#if 0
	for(i=0; i<p; i++){
		for(j=0; j<=i; j++){
			if(i==j){
				a11[i*p+j] = -wip(W[i], W[j], d, m) + sum(W[j], m);
				for(k=q[j]; k<q[j+1]; k++){
					for(l=q[j]; l<=k; l++){
						a22[k*Q+l] = -wip(X[k], X[l], d, m) + sum(X[k], m)*sum(X[l], m)/sum(W[j], m);
						b[k*Q+l] = ip(X[k], X[l], m) - sum(X[k], m)*sum(X[l], m)/sum(W[j], m);
					}
				}
			}else{
				a11[i*p+j] = -wip(W[i], W[j], d, m);
				for(k=q[i]; k<q[i+1]; k++){
					for(l=q[j]; l<q[j+1]; l++){
						a22[k*Q+l] = -wip(X[k], X[l], d, m);
						b[k*Q+l] = 0;
					}
				}
			}
		}
		for(j=0; j<p; j++){
			if(i==j){
				for(k=q[i]; k<q[i+1]; k++){
					a12[k*p+j] = wip(W[j], X[k], d, m) - ip(W[j], X[k], m);
				}
			}else{
				for(k=q[i]; k<q[i+1]; k++){
					a12[k*p+j] = wip(W[j], X[k], d, m);
				}
			}
		}
	}
#else
	for(i=0; i<p; i++){
		rtemp=r;
		qordered[i] = rtemp;
		s=0;
		for(j=0; j<=i; j++){
			stemp=s;
			r=rtemp;
			if(i>j){
				a11[i*p+j] = -wip(W[i], W[j], d, m);
				for(k=q[i]; k<q[i+1]; k++){
					if(diagb[k] > 0.0){
						//printf("%ld=%ld\n",r,k);
						s=stemp;
						for(l=q[j]; l<q[j+1]; l++){
							//printf(" %ld=%ld\n",s,l);
							if(diagb[l] > 0.0){
								//printf(" %ld=%ld\n",s,l);
								//printf(" %ld=%ld\n",r,k);
								
								a22[r*Q+s] = -wip(X[k], X[l], d, m);
								b[r*Q+s] = 0;
								//a22[k*Q+l] = -wip(X[k], X[l], d, m);
								//b[k*Q+l] = 0;
								s++;
							}
						}
						r++;
					}
				}
			}else{
				a11[i*p+j] = -wip(W[i], W[j], d, m) + sum(W[j], m);
				for(k=q[i]; k<q[i+1]; k++){
					if(diagb[k] > 0.0){
						//printf("%ld=%ld\n",r,k);
						s=stemp;
						for(l=q[j]; l<=k; l++){
							//printf(" %ld=%ld\n",s,l);
							if(diagb[l] > 0.0){
								
								
								a22[r*Q+s] = -wip(X[k], X[l], d, m) + sum(X[k], m)*sum(X[l], m)/sum(W[j], m);
								b[r*Q+s] = wip(X[k], X[l], W[j], m) - sum(X[k], m)*sum(X[l], m)/sum(W[j], m);
								//a22[r*Q+l] = -wip(X[k], X[l], d, m) + sum(X[k], m)*sum(X[l], m)/sum(W[j], m);
								//b[k*Q+l] = ip(X[k], X[l], m) - sum(X[k], m)*sum(X[l], m)/sum(W[j], m);
								s++;
							}
						}
						r++;
						//printf("r=%ld\n", r);
						//printf("j=%ld\n", j);
					}
				}
			}
		}
		for(j=0; j<p; j++){
			r=rtemp;
			if(i==j){
				for(k=q[i]; k<q[i+1]; k++){
					if(diagb[k] > 0.0){
						a12[r*p+j] = wip(W[j], X[k], d, m) - sum(X[k], m); //ip(W[j], X[k], m);
						r++;
					}
				}
			}else{
				for(k=q[i]; k<q[i+1]; k++){
					if(diagb[k] > 0.0){
						a12[r*p+j] = wip(W[j], X[k], d, m);
						r++;
					}
				}
			}
		}
	}
#endif
	qordered[i] = r;
	//printf("qordered=");
	//printVector(qordered,p+1,"%d");
	
	
	//printMatrix(a11, p, p, p);
	//printMatrix(a12, p, Q, p);
	//printMatrix(a22, Q, Q, Q);
	ginv(a11, p);
	//printMatrix(a11, p, p, p);
	ca(a11, a12, a22, p, Q);
	//printMatrix(a22, Q, Q, Q);
	//printMatrix(b, Q, Q, Q);
	//dsygv_(&itype, &jobz, &uplo, &nn, a22, &lda, b, &ldb, &ev, &work, &lwork, &info);
	if(lo == 0){
		Dsygv1(a22, b, Q, R, beta, dim, &maxev);
	}else{
		integer orderedOrig[Q+1]; //   2  3  4  5  7 8  10 11 12   ....   ** ** **
		integer orderedFlag[Q+1]; //  -1 -1 -1 -1   ....
		lc=0;
		//getOrdered(ordered, lo, q, orderedOrig);
		for(i=0;i<lo;i++){
			for(j=q[ordered[i]-1]; j<q[ordered[i]]; j++){
				//printf("%ld ", j);
				//printf("\n");
				orderedOrig[lc] = j;
				orderedFlag[lc] = 0;
				//printf("%ld ", orderedOrig[lc]);
				//printf("\n");
				lc++;
			}
		}
		rtemp=0;
		j=0;
		for(i=0; i<Q; i++){
			if(diagb[i] == 0.0){
				rtemp++;
			}
			if((i==orderedOrig[j]) && j<lc){
				orderedOrig[j]-=rtemp;
				j++;
			}
		}
		lc=j;
		//printVector(orderedOrig, lc, "%ld");
		//printVector(orderedFlag, lc, "%ld");
		
		/*
		Dsygv2(a22, b, &depth, qordered, Q, R, &maxev, beta, lo, ordered, orderedOrig, orderedFlag, lc, dim, &eflag,
		&itype, &jobz, &uplo, &lda, &ldb, &lwork, &info,
		work, ev, suba, subb, orderedCol, betaTemp, &tolerateDepth);
		*/
		
		// qordered is irrelevant to the ordered category.
		
		//Dsygv1(a22, b, Q, R, beta, dim, &maxev);
		Dsygv5(a22, b, qordered, Q, R, &maxev, beta, lo, ordered, orderedOrig, orderedFlag, lc, dim);
	}
	//double grad[Q+1];
	//gradient(a22, b, Q, R, &maxev, beta, grad);
	//printVector2(grad, Q, "%f");
	//printf("info=%d\n", info);
	
	//printf("maxev=%f\n", maxev);
	//printVector2(beta, Q, "%f");
	
	for(i=0; i<n; i++){
		y[i] = 0.0;
	}
	(*mev) = maxev;
	rtemp=0;
	r=0;
	for(j=0; j<p; j++){
		alpha = 0.0;
		for(k=0; k<R; k++){
			alpha += a12[p*k+j]*beta[k];
		}
		rtemp = r;
		//printf("%d  ",j);
		for(i=0; i<n; i++){
			r=rtemp;
			y[(j+1)*n+i] = 0.0;
			for(k=q[j]; k<q[j+1]; k++){
				if(diagb[k] > 0.0){
					y[(j+1)*n+i] += xx[k*n+i]*beta[r];
					if(i==0){
						//printf("%f<>",beta[r]);
					}
					r++;
				}
			}
			
			y[(j+1)*n+i] += alpha;
			y[i] += y[(j+1)*n+i]*w[j*n+i]/d[i];
		}
		//printf("\n");
	}
}
