\name{DandD}
\alias{DandD}
\title{Load DandD Instance}
\description{
   Fanction to load the DandD instance and Create \code{dad} object
}
\usage{
   DandD(file="", prefix="http://www.stat.math.keio.ac.jp/DandDIII/Examples/",
			 radix=TURE, language="English")
}
\arguments{
  \item{file}{character. a file name of a DandD instance.}
  \item{prefix}{A character string naming the URL without ``filename''.}
  \item{radix}{logical,  indicating whether or not radix structures should be maintained.}
  \item{language}{character. The language of the DandD instance(currently ignored).}
}
\value{
 \code{dad} object.
}
\details{
\code{DandD} loads a DandD instance and returns the \code{"dad"} object.
The dad object is a list of "Data Structure" elements that consist of
several "Data Vector" components. At the time of loading, a "Data Vector" component is 
expressed in a function call instead of actual data values, and 
it can be obtained later with the function \code{eval}.
If necessary, "Data Structure" can be coerced to the \code{data.frame} object simply with 
the function \code{data.frame}.

In addition, "Data Vector" components are attached to the R search path as a promise 
with the function \code{attachDandD} when printing dad object. 
By attaching to the search path, it is possible to refer to the "Data Vector" components 
by their names alone.

A \code{"dad"} object has its own \code{\link{plot}} method which
generates the best graphics of data by using rich attributes.

If \code{DandD} is called with "radix=FALSE", each "Radix Structure" consists of  
several "Data Vector" components are combined into one "Data Vector" component, 
see also \code{combineRadix}.
}
\seealso{
 \code{\link{openConn}}
 \code{\link{closeConn}}
 \code{\link{radix}}
}

\examples{\dontrun{

acid=DandD("AcidRain.dad")   # load a DandD instance.
acid                         # print the data structure name(s) and the data vector name(s)
acid$AcidRan$SlftCcn         # call object
eval(acid$AcidRan$SlftCcn)   # get actual values.
SlftCcn                      # access the promise object attached to the R search path.
acid.d=data.frame(acid[[1]]) # coerce to data frame.
acid.d
plot(acid)                   # generate plots
}
}
\keyword{dad}
\keyword{DandD}
\keyword{open}
